@@ -35,6 +35,7 @@ const DOCK_SIZE_KEY = 'size';
 const DOCK_HIDE_KEY = 'autohide';
 const DOCK_EFFECTHIDE_KEY = 'hide-effect';
 const DOCK_AUTOHIDE_ANIMATION_TIME_KEY = 'hide-effect-duration';
+const DOCK_MONITOR_KEY = 'monitor';
 
 // Keep enums in sync with GSettings schemas
 const PositionMode = {
@@ -50,6 +51,32 @@ const AutoHideEffect = {
 
 const DND_RAISE_APP_TIMEOUT = 500;
 
+// Utility function to make the dock clipped to the primary monitor
+function updateClip(actor, monitorNumber) {
+    let monitor;
+    if (monitorNumber > -1 && monitorNumber < Main.layoutManager.monitors.length)
+	monitor = Main.layoutManager.monitors[monitorNumber];
+    else
+	monitor = Main.layoutManager.primaryMonitor;
+
+    let allocation = actor.allocation;
+
+    // Here we implicitly assume that the stage and actor's parent
+    // share the same coordinate space
+    let clip = new Clutter.ActorBox({ x1: Math.max(monitor.x, allocation.x1),
+				      y1: Math.max(monitor.y, allocation.y1),
+				      x2: Math.min(monitor.x + monitor.width, allocation.x2),
+				      y2: Math.min(monitor.y + monitor.height, allocation.y2) });
+    // Translate back into actor's coordinate space
+    clip.x1 -= actor.x;
+    clip.x2 -= actor.x;
+    clip.y1 -= actor.y;
+    clip.y2 -= actor.y;
+
+    // Apply the clip
+    actor.set_clip(clip.x1, clip.y1, clip.x2-clip.x1, clip.y2 - clip.y1);
+}
+
 /*************************************************************************************/
 /**** start resize's Dock functions                                  *****************/
 /*************************************************************************************/
@@ -57,7 +84,10 @@ function hideDock_size () {
     if (!this._hideable)
         return;
 
-    let monitor = Main.layoutManager.primaryMonitor
+    let monitor = Main.layoutManager.primaryMonitor;
+    if (this._displayMonitor > -1 && this._displayMonitor < Main.layoutManager.monitors.length) {
+      monitor = Main.layoutManager.monitors[this._displayMonitor];
+    }
     let position_x = monitor.x;
     let height = (this._nicons)*(this._item_size + this._spacing) + 2*this._spacing;
     let width = this._item_size + 4*this._spacing;
@@ -79,6 +109,8 @@ function hideDock_size () {
             }
             this.actor.set_position (position_x,monitor.y+(monitor.height-height)/2);
             this.actor.set_size(width,height);
+
+	    updateClip(this.actor, this._displayMonitor);
         },
     });
 
@@ -87,6 +119,9 @@ function hideDock_size () {
 
 function showDock_size () {
     let monitor = Main.layoutManager.primaryMonitor;
+    if (this._displayMonitor > -1 && this._displayMonitor < Main.layoutManager.monitors.length) {
+      monitor = Main.layoutManager.monitors[this._displayMonitor];
+    }
     let height = (this._nicons)*(this._item_size + this._spacing) + 2*this._spacing;
     let width = this._item_size + 4*this._spacing;
     let position_x = monitor.x;
@@ -108,6 +143,8 @@ function showDock_size () {
             }
             this.actor.set_position (position_x, monitor.y+(monitor.height-height)/2);
             this.actor.set_size(width,height);
+
+	    updateClip(this.actor, this._displayMonitor);
         }
     });
 
@@ -124,7 +161,11 @@ function showEffectAddItem_size () {
         height: height,
         width: width,
         time: this._settings.get_double(DOCK_AUTOHIDE_ANIMATION_TIME_KEY),
-        transition: 'easeOutQuad'
+        transition: 'easeOutQuad',
+	onUpdate: function (monitor) {
+	    updateClip(this, monitor);
+	},
+	onUpdateParams: [this._displayMonitor]
     });
 }
 
@@ -137,6 +178,9 @@ function hideDock_scale () {
 
     this._item_size = this._settings.get_int(DOCK_SIZE_KEY);
     let monitor = Main.layoutManager.primaryMonitor;
+    if (this._displayMonitor > -1 && this._displayMonitor < Main.layoutManager.monitors.length) {
+      monitor = Main.layoutManager.monitors[this._displayMonitor];
+    }
     let cornerX = 0;
     let height = this._nicons*(this._item_size + this._spacing) + 2*this._spacing;
     let width = this._item_size + 4*this._spacing;
@@ -157,7 +201,11 @@ function hideDock_scale () {
         width: width,
         scale_x: 0.025,
         time: this._settings.get_double(DOCK_AUTOHIDE_ANIMATION_TIME_KEY),
-        transition: 'easeOutQuad'
+        transition: 'easeOutQuad',
+	onUpdate: function(monitor) {
+	    updateClip(this, monitor);
+	},
+	onUpdateParams: [this._displayMonitor]
     });
 
     this._hidden = true;
@@ -166,6 +214,9 @@ function hideDock_scale () {
 function showDock_scale () {
     this._item_size = this._settings.get_int(DOCK_SIZE_KEY);
     let monitor = Main.layoutManager.primaryMonitor;
+    if (this._displayMonitor > -1 && this._displayMonitor < Main.layoutManager.monitors.length) {
+      monitor = Main.layoutManager.monitors[this._displayMonitor];
+    }
     let position_x = monitor.x;
     let height = this._nicons*(this._item_size + this._spacing) + 2*this._spacing;
     let width = this._item_size + 4*this._spacing;
@@ -185,7 +236,11 @@ function showDock_scale () {
         width: width,
         scale_x: 1,
         time: this._settings.get_double(DOCK_AUTOHIDE_ANIMATION_TIME_KEY),
-        transition: 'easeOutQuad'
+        transition: 'easeOutQuad',
+	onUpdate: function(monitor) {
+	    updateClip(this, monitor);
+	},
+	onUpdateParams: [this._displayMonitor]
     });
 
     this._hidden = false;
@@ -193,6 +248,9 @@ function showDock_scale () {
 
 function showEffectAddItem_scale () {
     let monitor = Main.layoutManager.primaryMonitor;
+    if (this._displayMonitor > -1 && this._displayMonitor < Main.layoutManager.monitors.length) {
+      monitor = Main.layoutManager.monitors[this._displayMonitor];
+    }
     let height = this._nicons*(this._item_size + this._spacing) + 2*this._spacing;
     let width = this._item_size + 4*this._spacing;
 
@@ -201,7 +259,11 @@ function showEffectAddItem_scale () {
         height: height,
         width: width,
         time: this._settings.get_double(DOCK_AUTOHIDE_ANIMATION_TIME_KEY),
-        transition: 'easeOutQuad'
+        transition: 'easeOutQuad',
+	onUpdate: function(monitor) {
+	    updateClip(this, monitor);
+	},
+	onUpdateParams: [this._displayMonitor]
     });
 }
 
@@ -214,6 +276,9 @@ function hideDock_move () {
 
     this._item_size = this._settings.get_int(DOCK_SIZE_KEY);
     let monitor = Main.layoutManager.primaryMonitor;
+    if (this._displayMonitor > -1 && this._displayMonitor < Main.layoutManager.monitors.length) {
+      monitor = Main.layoutManager.monitors[this._displayMonitor];
+    }
     let cornerX = 0;
     let height = this._nicons*(this._item_size + this._spacing) + 2*this._spacing;
     let width = this._item_size + 4*this._spacing;
@@ -233,7 +298,11 @@ function hideDock_move () {
         width: width,
         height: height,
         time: this._settings.get_double(DOCK_AUTOHIDE_ANIMATION_TIME_KEY),
-        transition: 'easeOutQuad'
+        transition: 'easeOutQuad',
+	onUpdate: function(monitor) {
+	    updateClip(this, monitor);
+	},
+	onUpdateParams: [this._displayMonitor]
     });
 
     this._hidden = true;
@@ -242,6 +311,9 @@ function hideDock_move () {
 function showDock_move () {
     this._item_size = this._settings.get_int(DOCK_SIZE_KEY);
     let monitor = Main.layoutManager.primaryMonitor;
+    if (this._displayMonitor > -1 && this._displayMonitor < Main.layoutManager.monitors.length) {
+      monitor = Main.layoutManager.monitors[this._displayMonitor];
+    }
     let position_x = monitor.x;
     let height = this._nicons*(this._item_size + this._spacing) + 2*this._spacing;
     let width = this._item_size + 4*this._spacing;
@@ -260,24 +332,35 @@ function showDock_move () {
         width: width,
         height: height,
         time: this._settings.get_double(DOCK_AUTOHIDE_ANIMATION_TIME_KEY),
-        transition: 'easeOutQuad'
+        transition: 'easeOutQuad',
+	onUpdate: function(monitor) {
+	    updateClip(this, monitor);
+	},
+	onUpdateParams: [this._displayMonitor]
     });
 
     this._hidden = false;
 }
 
 function showEffectAddItem_move () {
-        let monitor = Main.layoutManager.primaryMonitor;
-        let height = this._nicons*(this._item_size + this._spacing) + 2*this._spacing;
-        let width = this._item_size + 4*this._spacing;
+    let monitor = Main.layoutManager.primaryMonitor;
+    if (this._displayMonitor > -1 && this._displayMonitor < Main.layoutManager.monitors.length) {
+        monitor = Main.layoutManager.monitors[this._displayMonitor];
+    }
+    let height = this._nicons*(this._item_size + this._spacing) + 2*this._spacing;
+    let width = this._item_size + 4*this._spacing;
 
-        Tweener.addTween(this.actor, {
-                y: monitor.y + (monitor.height-height)/2,
-                height: height,
-                width: width,
-                time: this._settings.get_double(DOCK_AUTOHIDE_ANIMATION_TIME_KEY),
-                transition: 'easeOutQuad'
-        });
+    Tweener.addTween(this.actor, {
+        y: monitor.y + (monitor.height-height)/2,
+        height: height,
+        width: width,
+        time: this._settings.get_double(DOCK_AUTOHIDE_ANIMATION_TIME_KEY),
+        transition: 'easeOutQuad',
+	onUpdate: function(monitor) {
+	    updateClip(this, monitor);
+	},
+	onUpdateParams: [this._displayMonitor]
+    });
 }
 
 const Dock = new Lang.Class({
@@ -294,13 +377,16 @@ const Dock = new Lang.Class({
         this._settings = Convenience.getSettings();
         this._hidden = false;
         this._hideable = this._settings.get_boolean(DOCK_HIDE_KEY);
+        this._displayMonitor = this._settings.get_int(DOCK_MONITOR_KEY);
 
         this._spacing = 4;
         this._item_size = this._settings.get_int(DOCK_SIZE_KEY);
         this._nicons = 0;
         this._selectEffectFunctions(this._settings.get_enum(DOCK_EFFECTHIDE_KEY));
 
-        this.actor = new St.BoxLayout({ name: 'dock', vertical: true, reactive: true });
+        let [_x, _y, _w, _h] = this.get_start_position();
+        this.actor = new St.BoxLayout({ name: 'dock', vertical: true, reactive: true,
+                                        x: _x, y: _y, width: _w, height: _h });
 
         this._grid = new Shell.GenericContainer();
         this.actor.add(this._grid, { expand: true, y_align: St.Align.START });
@@ -331,6 +417,10 @@ const Dock = new Lang.Class({
         //hidden
         this._settings.connect('changed::'+DOCK_POSITION_KEY, Lang.bind(this, this._redisplay));
         this._settings.connect('changed::'+DOCK_SIZE_KEY, Lang.bind(this, this._redisplay));
+        this._settings.connect('changed::'+DOCK_MONITOR_KEY, Lang.bind(this, function (){
+            this._displayMonitor = this._settings.get_int(DOCK_MONITOR_KEY);
+            this._redisplay();
+        }));
 
         this._settings.connect('changed::'+DOCK_HIDE_KEY, Lang.bind(this, function (){
                 Main.layoutManager.removeChrome(this.actor);
@@ -376,6 +466,27 @@ const Dock = new Lang.Class({
         this._hideDock();
     },
 
+    get_start_position: function() {
+        let item_size = this._settings.get_int(DOCK_SIZE_KEY);
+        let monitor = Main.layoutManager.primaryMonitor;
+        if (this._displayMonitor > -1 && this._displayMonitor < Main.layoutManager.monitors.length) {
+          monitor = Main.layoutManager.monitors[this._displayMonitor];
+        }
+        let position_x = monitor.x;
+        let width = item_size + 4 * this._spacing;
+
+        switch (this._settings.get_enum(DOCK_POSITION_KEY)) {
+        case PositionMode.LEFT:
+            position_x=monitor.x - 2 * this._spacing;
+            break;
+        case PositionMode.RIGHT:
+        default:
+            position_x=monitor.x + (monitor.width - item_size - 2 * this._spacing);
+        }
+
+        return [ position_x, monitor.y, width, monitor.height ];
+    },
+
     destroy: function() {
         if (this._installedChangedId) {
             this._appSystem.disconnect(this._installedChangedId);
