@@ -301,13 +301,13 @@ function enable() {
     workspaceInjections['_calculateWindowTransformationsNatural'] = undefined;
 
     /**
-     * positionWindows:
+     * _realPositionWindows:
      * @flags:
      *  INITIAL - this is the initial positioning of the windows.
      *  ANIMATE - Indicates that we need animate changing position.
      */
-    workspaceInjections['positionWindows'] = Workspace.Workspace.prototype.positionWindows;
-    Workspace.Workspace.prototype.positionWindows = function(flags) {
+    workspaceInjections['positionWindows'] = Workspace.Workspace.prototype._realPositionWindows;
+    Workspace.Workspace.prototype._realPositionWindows = function(flags) {
         if (this._repositionWindowsId > 0) {
             Mainloop.source_remove(this._repositionWindowsId);
             this._repositionWindowsId = 0;
